@@ -1,27 +1,30 @@
 from time import sleep
 import pandas as pd
-from urllib3.exceptions import NewConnectionError
 from requests_html import HTMLSession
 import logging
+import sys
+
 
 from .base_scrape import BaseScrape
+from ..utils.header_vars import random_header_vars
 
 
 class Scrape(BaseScrape):
     def __init__(self,
-        post_process_func:callable,
-        post_process_kwargs:dict={},
-        fetch_error_handler:callable=None,
-        use_proxy:bool=False,
-        proxy:str=None,
-        pac_url:str=None,
-        consecutive_error_limit:int=100,
-        attempt_limit:int=5,
-        rest_between_attempts:bool=True,
-        rest_wait:int=60
-        ):
+                 post_process_func: callable,
+                 post_process_kwargs: dict = {},
+                 fetch_error_handler: callable = None,
+                 use_proxy: bool = False,
+                 proxy: str = None,
+                 pac_url: str = None,
+                 consecutive_error_limit: int = 100,
+                 attempt_limit: int = 5,
+                 rest_between_attempts: bool = True,
+                 rest_wait: int = 60,
+                 randomise_headers: bool = False
+                 ):
         """Class for scrapping webpages
-        
+
         args:
         ----
         post_process_func - callable - for processing html
@@ -40,8 +43,9 @@ class Scrape(BaseScrape):
         rest_between_attempts - bool:True - should the program rest between scrapes
         rest_wait - int:60 - how long should the program rest for ONLY VALID IF
             REST_BETWEEN_SCRAPES IS TRUE
+        randomise_headers - bool:False - should the headers be randomised after each request
         """
-        #Init super
+        # Init super
         super().__init__(
             use_proxy=use_proxy,
             proxy=proxy,
@@ -49,14 +53,15 @@ class Scrape(BaseScrape):
         )
         self.post_process = post_process_func
         self.post_process_kwargs = post_process_kwargs
-        self.headers = {}
+        self.randomise_headers = randomise_headers
+        self.headers = random_header_vars(self.header_vars)
         self.fetch_error_handler = fetch_error_handler
         self.session = HTMLSession()
-        #Define allowed errors
+        # Define allowed errors
         self.acceptable_errors = ()
         self.consecutive_error_limit = consecutive_error_limit
         self.consecutive_error_count = 0
-        #Define criteria for looping multiple attempts
+        # Define criteria for looping multiple attempts
         self.attempt_limit = attempt_limit
         self.rest_between_attempts = rest_between_attempts
         self.rest_wait = rest_wait
@@ -64,13 +69,13 @@ class Scrape(BaseScrape):
         self.cur_err = None
 
     def _proxy(self):
-        #Start the pac session
+        # Start the pac session
         self._get_pac_session()
         self.session = self.pac_session
-        
+
     def _fetch(self, url):
         """Function to fetch HTML from url
-        
+
         args:
         ----
         url - str - url to be requested
@@ -80,45 +85,50 @@ class Scrape(BaseScrape):
         list
         """
         resp = None
-        #Make the request
+        # Make the request
         try:
             if url:
                 resp = self.session.get(url, headers=self.headers)
                 if resp.status_code == 200:
                     html = resp.text
-                    func_resp = self.post_process(html=html, resp=resp, **self.post_process_kwargs)
+                    func_resp = self.post_process(
+                        html=html, resp=resp, **self.post_process_kwargs)
                 else:
                     func_resp = None
-                #Reset self.acceptable_error_count if all goes fine
+                # Reset self.acceptable_error_count if all goes fine
                 self.consecutive_error_count = 0
-                return {"url":url, "func_resp":func_resp, "status":resp.status_code, "error":None}
+                return {"url": url, "func_resp": func_resp, "status": resp.status_code, "error": None}
         except Exception as e:
-            #Set the current error - increment if the same error
+            # Set the current error - increment if the same error
             if type(e) == self.cur_err:
                 self.consecutive_error_count += 1
             else:
                 self.cur_err = type(e)
                 self.consecutive_error_count = 1
-            #Check if acceptabe error limit has been reached
+            # Check if acceptabe error limit has been reached
             # this prevents functions from carrying on after a site has started blocking calls
             if self.consecutive_error_count >= self.consecutive_error_limit:
-                logging.warning(f"Consecutive error limit reached - {e} - consecutive count at {self.consecutive_error_count}/{self.consecutive_error_limit}")
-            #Check for error handler
+                logging.warning(
+                    f"Consecutive error limit reached - {e} - consecutive count at {self.consecutive_error_count}/{self.consecutive_error_limit}")
+            # Check for error handler
             if self.fetch_error_handler:
-                logging.info(f"Error passed to {self.fetch_error_handler.__name__}")
-                #Run the error handler
+                logging.info(
+                    f"Error passed to {self.fetch_error_handler.__name__}")
+                # Run the error handler
                 self.fetch_error_handler(url, e)
-            #Check if acceptable error
+            # Check if acceptable error
             if type(e) in self.acceptable_errors:
-                logging.error(f"Acceptable error in request or post processing {url} - {e}")
-            #Raise error
+                logging.error(
+                    f"Acceptable error in request or post processing {url} - {e}")
+            # Raise error
             else:
-                logging.error(f"Unhandled error in request or post processing {url} - {e}")
+                logging.error(
+                    f"Unhandled error in request or post processing {url} - {e}")
                 if f"{e}" == "":
                     raise e
-            return {"url":url, "func_resp":None, "status":None, "error":e}
+            return {"url": url, "func_resp": None, "status": None, "error": e}
 
-    def _increment_attempts(self, scraped:bool, urls:list=None):
+    def _increment_attempts(self, scraped: bool, urls: list = None):
         if urls:
             filter_df = self.tracker_df.url.isin(urls)
             self.tracker_df.loc[filter_df, "scraped"] = scraped
@@ -127,11 +137,11 @@ class Scrape(BaseScrape):
             self.tracker_df["scraped"] = scraped
             self.tracker_df["attempts"] += 1
 
-    #run from terminal
-    def scrape_all(self, urls:list):
+    # run from terminal
+    def scrape_all(self, urls: list):
         """"Function scraping html from urls and passing 
         them through the post processing function
-        
+
         args:
         ----
         urls - list - the pages to be scraped
@@ -148,12 +158,12 @@ class Scrape(BaseScrape):
         self.start_job()
         if self.use_proxy:
             self._proxy()
-        #Establish urls
+        # Establish urls
         if not len(urls):
             return []
-        #Set a dataframe for tracking the url attempts
+        # Set a dataframe for tracking the url attempts
         self.tracker_df = pd.DataFrame([
-            {"url":u, "scraped":False, "attempts":0}
+            {"url": u, "scraped": False, "attempts": 0}
             for u in urls
         ])
         resps = []
@@ -162,34 +172,37 @@ class Scrape(BaseScrape):
         while len(scrape_urls):
             self.reset_pages_scraped()
             self.total_to_scrape = len(scrape_urls)
-            #Runt he scrapes
+            # Runt he scrapes
             scrape_resps = []
             for url in urls:
                 scrape_resps.append(self._fetch(url))
                 self.increment_pages_scraped()
-            #Add scrape_resps to resps
+                # Regenerate headers
+                if self.randomise_headers:
+                    self.headers = random_header_vars(self.header_vars)
+            # Add scrape_resps to resps
             resps.extend(scrape_resps)
-            #Get scraped urls
+            # Get scraped urls
             scraped_urls = [
                 r["url"] for r in resps
                 if not r["error"]
-                ]
-            #Increment attempts count on each scraped url
+            ]
+            # Increment attempts count on each scraped url
             self._increment_attempts(True, scrape_urls)
-            #Get errored urls
+            # Get errored urls
             errored_urls = [
                 r["url"] for r in resps
                 if r["error"]
-                ]
-            #Increment attempts count on each attempted but failed (IE had an error 
+            ]
+            # Increment attempts count on each attempted but failed (IE had an error
             # but not cancelled)
             self._increment_attempts(False, errored_urls)
-            #Remove scraped urls from scrape_urls
+            # Remove scraped urls from scrape_urls
             scrape_urls = [
                 u for u in scrape_urls
                 if u not in scraped_urls
             ]
-            #Remove urls where too many attempts have been made
+            # Remove urls where too many attempts have been made
             failed_urls = self.tracker_df[
                 self.tracker_df.attempts >= self.attempt_limit
             ].url.to_list()
@@ -197,27 +210,28 @@ class Scrape(BaseScrape):
                 u for u in scrape_urls
                 if u not in failed_urls
             ]
-            logging.info(f"Scraping round {i} complete, in this round - {len(scraped_urls)} urls scrapped, {len(errored_urls)} urls errored, {len(errored_urls) - len(scrape_urls)} urls failed, {len(scrape_urls)} remain unscrapped")
-            #Increment the loop number 
+            logging.info(
+                f"Scraping round {i} complete, in this round - {len(scraped_urls)} urls scrapped, {len(errored_urls)} urls errored, {len(errored_urls) - len(scrape_urls)} urls failed, {len(scrape_urls)} remain unscrapped")
+            # Increment the loop number
             i += 1
-            #Sleep before running again
+            # Sleep before running again
             # - shutdown must have been initiated
             # - there must still be urls to scrape
-            # - the rest between attempt flag must be set to True 
+            # - the rest between attempt flag must be set to True
             if len(scrape_urls) \
-                and self.rest_between_attempts:
+                    and self.rest_between_attempts:
                 logging.info(f"Sleeping for {self.rest_wait} seconds")
                 sleep(self.rest_wait)
         logging.info(f"Scraping complete {len(failed_urls)} urls failed")
-        #end the job
+        # end the job
         self.end_job()
         return resps
 
-    #run from terminal
-    def scrape_one(self, url:str):
+    # run from terminal
+    def scrape_one(self, url: str):
         """"Function scraping html from a single url and passing 
         it through the post processing function
-        
+
         args:
         ----
         url - str - the pages to be scraped
@@ -240,4 +254,4 @@ class Scrape(BaseScrape):
             elif self.rest_between_attempts:
                 logging.info(f"Sleeping for {self.rest_wait} seconds")
                 sleep(self.rest_wait)
-        return resp
\ No newline at end of file
+        return resp
